package webboard;

import bell.oauth.discord.domain.User;
import bell.oauth.discord.main.OAuthBuilder;
import bell.oauth.discord.main.Response;
import com.corundumstudio.socketio.SocketIOClient;
import misc.Utility;

import java.util.Properties;

public class Session {
    private final String CLIENT_ID;
    private final String CLIENT_SECRET;
    private final String[] CLIENT_SCOPE = new String[]{"identify"};
    private final String CLIENT_CALLBACK;
    private Properties webProperties;
    public final long CREATION_TIME;

    private SocketIOClient client;
    private OAuthBuilder builder;
    private String authURL;
    private String token;

    public Session(SocketIOClient socketIOClient){
        CREATION_TIME = System.currentTimeMillis();

        webProperties = new Properties();
        Utility.loadProperties("web.properties", webProperties);
        CLIENT_ID = webProperties.getProperty("oauthclientid");
        CLIENT_SECRET = webProperties.getProperty("oauthclientsecret");
        CLIENT_CALLBACK = webProperties.getProperty("oauthclientcallback");

        client = socketIOClient;
        builder = new OAuthBuilder(CLIENT_ID, CLIENT_SECRET)
                .setScopes(CLIENT_SCOPE)
                .setRedirectURI(CLIENT_CALLBACK);
        authURL = builder.getAuthorizationUrl(null);
    }

    public String getAuthURL(){
        return authURL;
    }

    public void setToken(String token){
        this.token = token;
    }

    public SocketIOClient getClient(){
        return client;
    }

    public long getUserID(){
        Response response = builder.exchange(token);
        if(response == Response.ERROR){
            return -1;
        }else{
            User user = builder.getUser();
            return Long.valueOf(user.getId());
        }
    }
}
